@@ -21,6 +21,7 @@
 #include <config.h>
 #include "system.h"
 
+#include <attribute.h>
 #include <error.h>
 #include <hash.h>
 #include <quotearg.h>
@@ -49,8 +50,7 @@ uniqstr_new (char const *str)
     {
       /* First insertion in the hash. */
       res = xstrdup (str);
-      if (!hash_insert (uniqstrs_table, res))
-        xalloc_die ();
+      hash_xinsert (uniqstrs_table, res);
     }
   return res;
 }
@@ -80,9 +80,7 @@ uniqstr_concat (int nargs, ...)
   va_end (args);
 
   *p = '\0';
-  uniqstr res = hash_insert (uniqstrs_table, str);
-  if (!res)
-    xalloc_die ();
+  uniqstr res = hash_xinsert (uniqstrs_table, str);
   if (res != str)
     free (str);
   return res;
@@ -117,12 +115,11 @@ uniqstr_print (uniqstr ustr)
 }
 
 static bool
-uniqstr_print_processor (void *ustr, void *null ATTRIBUTE_UNUSED)
+uniqstr_print_processor (void *ustr, void *null MAYBE_UNUSED)
 {
   return uniqstr_print (ustr);
 }
 
-
 int
 uniqstr_cmp (uniqstr l, uniqstr r)
 {
